package member

import (
	"app-cloudep-portal-api-gateway/internal/domain"
	"app-cloudep-portal-api-gateway/internal/svc"
	"app-cloudep-portal-api-gateway/internal/types"
	"context"
	"fmt"

	notificationRpc "code.30cm.net/digimon/proto-all/pkg/notification"

	ers "code.30cm.net/digimon/library-go/errors"
	accountRpc "code.30cm.net/digimon/proto-all/pkg/member"
	"github.com/matcornic/hermes/v2"

	"github.com/zeromicro/go-zero/core/logx"
)

type ForgetPasswordCodeLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewForgetPasswordCodeLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ForgetPasswordCodeLogic {
	return &ForgetPasswordCodeLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ForgetPasswordCodeLogic) ForgetPasswordCode(req *types.ForgetPasswordCodeReq) (*types.BaseResponse, error) {
	// 限制三分鐘內只可以發送一次
	rk := domain.GenerateVerifyCodeRedisKey.With(fmt.Sprintf("%s-%d", req.Account, domain.SendVerifyCodeTypeForgetPassword)).ToString()
	get, err := l.svcCtx.Redis.Get(rk)
	if err != nil {
		// Redis 錯誤，給予適當的提示（可以視情況返回錯誤或繼續）
		l.Logger.Errorf("Redis error: %v", err)
	}
	if get != "" {
		// 已經發送過驗證碼，返回提示
		return nil, ers.ArkInternal("verification code already sent, please wait before requesting again")
	}

	// 確認帳號是否已經註冊過
	accountResp, err := l.svcCtx.AccountRpc.GetUidByAccount(l.ctx, &accountRpc.GetUIDByAccountReq{
		Account: req.Account,
	})
	if err != nil {
		return nil, err
	}

	// 生成重置密碼驗證碼
	code, err := l.svcCtx.AccountRpc.GenerateRefreshCode(l.ctx, &accountRpc.GenerateRefreshCodeReq{
		Account:  req.Account,
		CodeType: domain.SendVerifyCodeTypeForgetPassword,
	})
	if err != nil {
		return nil, err
	}

	// 取得用戶資訊
	info, err := l.svcCtx.AccountRpc.GetUserInfo(l.ctx, &accountRpc.GetUserInfoReq{Uid: accountResp.Uid})
	if err != nil {
		return nil, err
	}

	// 準備驗證碼郵件
	nickName := info.Data.Uid
	if info.Data.NickName != nil {
		nickName = *info.Data.NickName
	}
	mailContent, title, err := ForgerZHTW(nickName, code.Data.VerifyCode)
	if err != nil {
		return nil, ers.InvalidFormat("failed to generate mail content: ", err.Error())
	}

	// 發送郵件
	_, err = l.svcCtx.NotificationRpc.SendMail(l.ctx, &notificationRpc.SendMailReq{
		Body:    mailContent,
		Subject: title,
		To:      req.Account,
		From:    l.svcCtx.Config.MailSender,
	})
	if err != nil {
		return nil, err
	}

	// 設置 Redis 鍵，並設置 3 分鐘的過期時間
	err = l.svcCtx.Redis.Set(rk, code.Data.VerifyCode)
	if err != nil {
		return nil, ers.ArkInternal(err.Error())
	}
	err = l.svcCtx.Redis.Expire(rk, 180)
	if err != nil {
		return nil, ers.ArkInternal(err.Error())
	}

	// 返回成功響應
	return &types.BaseResponse{
		Status: types.Status{
			Code:    domain.SuccessCode,
			Message: domain.SuccessMsg,
		},
	}, nil
}

type generateForgetPasswordMailBodyReq struct {
	ProductName      string
	ProductLink      string
	ProductLogo      string
	ProductCopyright string

	BodyName      string
	BodyIntros    []string
	BodyActions   []hermes.Action
	BodyOutros    []string
	BodySignature string
}

func ForgerZHTW(username, verifyCode string) (body, title string, err error) {
	req := generateForgetPasswordMailBodyReq{
		ProductName:      "Digimon 團隊",
		ProductLink:      "https://code.30cm.net",
		ProductLogo:      "https://storage.googleapis.com/netpute-qa-public/logo_dark.png",
		ProductCopyright: "© 2024 Digimon Inc. 版權所有",

		BodyName: username,
		BodyIntros: []string{
			"您收到此電子郵件是因為我們收到了針對 Digimon 帳戶的密碼重置請求。",
		},
		BodyActions: []hermes.Action{
			{
				Instructions: "請複製您的驗證碼，到網頁重置",
				InviteCode:   verifyCode,
			},
		},
		BodyOutros: []string{
			"如果您不要求重設密碼，則無需您採取進一步的措施。",
		},
		BodySignature: "無所不在的即時遊戲社群平台",
	}
	pr, bo := mustForgetPasswordMail(req)

	body, err = genMailBody(forgetPasswordTemplate{pr, bo})

	return body, "Digimon 平台，重設密碼驗證信", err
}

func mustForgetPasswordMail(req generateForgetPasswordMailBodyReq) (hermes.Product, hermes.Body) {
	product := hermes.Product{
		// Appears in header & footer of e-mails
		Name:      req.ProductName,
		Link:      req.ProductLink,
		Logo:      req.ProductLogo,
		Copyright: req.ProductCopyright,
	}

	body := hermes.Body{
		Name:      req.BodyName,
		Intros:    req.BodyIntros,
		Actions:   req.BodyActions,
		Outros:    req.BodyOutros,
		Signature: req.BodySignature,
	}

	return product, body
}

type forgetPasswordTemplate struct {
	Product hermes.Product
	Body    hermes.Body
}

func genMailBody(req forgetPasswordTemplate) (body string, err error) {
	h := hermes.Hermes{
		Product: req.Product,
	}
	email := hermes.Email{
		Body: req.Body,
	}
	emailBody, err := h.GenerateHTML(email)
	if err != nil {
		return "", err
	}
	return emailBody, nil
}
